
# 
# Cpppo -- Communication Protocol Python Parser and Originator
# 
# Copyright (c) 2013, Hard Consulting Corporation.
# 
# Cpppo is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.  See the LICENSE file at the top of the source tree.
# 
# Cpppo is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
# 

# 
# cpppo.server.enip.weather
# 
# Access a Tag-named location's current temperature via EtherNet/IP.
# 
#     $ python -m cpppo.server.enip.weather London=REAL &
#     $ python -m cpppo.server.enip.client --print London
#               London              == [15.319999694824219]: 'OK'
# 
from __future__ import absolute_import
from __future__ import print_function
from __future__ import division

__author__                      = "Perry Kundert"
__email__                       = "perry@hardconsulting.com"
__copyright__                   = "Copyright (c) 2013 Hard Consulting Corporation"
__license__                     = "Dual License: GPLv3 (or later) and Commercial (see LICENSE)"

import sys, logging, json
try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from cpppo.server.enip import device
from cpppo.server.enip.main import main

class Attribute_weather( device.Attribute ):
    def __getitem__( self, key ):
        try:
            url = "http://api.openweathermap.org/data/2.5/weather?units=metric&q=%s" % self.name 
            weather = json.loads( urlopen( url ).read() )
            return [ weather['main']['temp'] ]
        except Exception as exc:
            logging.warning( "Couldn't get temperature for %s: %s", self.name, exc )
            raise

    def __setitem__( self, key, value ):
        raise Exception( "Changing the weather isn't that easy..." )

sys.exit( main( attribute_class=Attribute_weather ))
